@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2023
+# Copyright (C) 2015-2024
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -18,23 +18,26 @@
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains an object that represents a Telegram Update."""
 
-from typing import TYPE_CHECKING, ClassVar, List, Optional
+from typing import TYPE_CHECKING, Final, List, Optional
 
 from telegram import constants
 from telegram._callbackquery import CallbackQuery
+from telegram._chatboost import ChatBoostRemoved, ChatBoostUpdated
 from telegram._chatjoinrequest import ChatJoinRequest
 from telegram._chatmemberupdated import ChatMemberUpdated
 from telegram._choseninlineresult import ChosenInlineResult
 from telegram._inline.inlinequery import InlineQuery
 from telegram._message import Message
+from telegram._messagereactionupdated import MessageReactionCountUpdated, MessageReactionUpdated
 from telegram._payment.precheckoutquery import PreCheckoutQuery
 from telegram._payment.shippingquery import ShippingQuery
 from telegram._poll import Poll, PollAnswer
 from telegram._telegramobject import TelegramObject
 from telegram._utils.types import JSONDict
+from telegram._utils.warnings import warn
 
 if TYPE_CHECKING:
-    from telegram import Bot, Chat, User  # noqa
+    from telegram import Bot, Chat, User
 
 
 class Update(TelegramObject):
@@ -46,7 +49,7 @@ class Update(TelegramObject):
     Note:
         At most one of the optional parameters can be present in any given update.
 
-    .. seealso:: :wiki:`Your First Bot <Extensions-–-Your-first-Bot>`
+    .. seealso:: :wiki:`Your First Bot <Extensions---Your-first-Bot>`
 
     Args:
         update_id (:obj:`int`): The update's unique identifier. Update identifiers start from a
@@ -58,11 +61,13 @@ class Update(TelegramObject):
         message (:class:`telegram.Message`, optional): New incoming message of any kind - text,
             photo, sticker, etc.
         edited_message (:class:`telegram.Message`, optional): New version of a message that is
-            known to the bot and was edited.
+            known to the bot and was edited. This update may at times be triggered by changes to
+            message fields that are either unavailable or not actively used by your bot.
         channel_post (:class:`telegram.Message`, optional): New incoming channel post of any kind
             - text, photo, sticker, etc.
         edited_channel_post (:class:`telegram.Message`, optional): New version of a channel post
-            that is known to the bot and was edited.
+            that is known to the bot and was edited. This update may at times be triggered by
+            changes to message fields that are either unavailable or not actively used by your bot.
         inline_query (:class:`telegram.InlineQuery`, optional): New incoming inline query.
         chosen_inline_result (:class:`telegram.ChosenInlineResult`, optional): The result of an
             inline query that was chosen by a user and sent to their chat partner.
@@ -72,7 +77,7 @@ class Update(TelegramObject):
         pre_checkout_query (:class:`telegram.PreCheckoutQuery`, optional): New incoming
             pre-checkout query. Contains full information about checkout.
         poll (:class:`telegram.Poll`, optional): New poll state. Bots receive only updates about
-            stopped polls and polls, which are sent by the bot.
+            manually stopped polls and polls, which are sent by the bot.
         poll_answer (:class:`telegram.PollAnswer`, optional): A user changed their answer
             in a non-anonymous poll. Bots receive new votes only in polls that were sent
             by the bot itself.
@@ -96,6 +101,39 @@ class Update(TelegramObject):
             receive these updates.
 
             .. versionadded:: 13.8
+
+        chat_boost (:class:`telegram.ChatBoostUpdated`, optional): A chat boost was added or
+            changed. The bot must be an administrator in the chat to receive these updates.
+
+            .. versionadded:: 20.8
+
+        removed_chat_boost (:class:`telegram.ChatBoostRemoved`, optional): A boost was removed from
+            a chat. The bot must be an administrator in the chat to receive these updates.
+
+            .. versionadded:: 20.8
+
+        message_reaction (:class:`telegram.MessageReactionUpdated`, optional): A reaction to a
+            message was changed by a user. The bot must be an administrator in the chat and must
+            explicitly specify :attr:`MESSAGE_REACTION` in the list of
+            :paramref:`telegram.ext.Application.run_polling.allowed_updates` to receive these
+            updates (see :meth:`telegram.Bot.get_updates`, :meth:`telegram.Bot.set_webhook`,
+            :meth:`telegram.ext.Application.run_polling` and
+            :meth:`telegram.ext.Application.run_webhook`). The update isn't received for reactions
+            set by bots.
+
+            .. versionadded:: 20.8
+
+        message_reaction_count (:class:`telegram.MessageReactionCountUpdated`, optional): Reactions
+            to a message with anonymous reactions were changed. The bot must be an administrator in
+            the chat and must explicitly specify :attr:`MESSAGE_REACTION_COUNT` in the list of
+            :paramref:`telegram.ext.Application.run_polling.allowed_updates` to receive these
+            updates (see :meth:`telegram.Bot.get_updates`, :meth:`telegram.Bot.set_webhook`,
+            :meth:`telegram.ext.Application.run_polling` and
+            :meth:`telegram.ext.Application.run_webhook`). The updates are grouped and can be sent
+            with delay up to a few minutes.
+
+            .. versionadded:: 20.8
+
     Attributes:
         update_id (:obj:`int`): The update's unique identifier. Update identifiers start from a
             certain positive number and increase sequentially. This ID becomes especially handy if
@@ -106,11 +144,13 @@ class Update(TelegramObject):
         message (:class:`telegram.Message`): Optional. New incoming message of any kind - text,
             photo, sticker, etc.
         edited_message (:class:`telegram.Message`): Optional. New version of a message that is
-            known to the bot and was edited.
+            known to the bot and was edited. This update may at times be triggered by changes to
+            message fields that are either unavailable or not actively used by your bot.
         channel_post (:class:`telegram.Message`): Optional. New incoming channel post of any kind
             - text, photo, sticker, etc.
         edited_channel_post (:class:`telegram.Message`): Optional. New version of a channel post
-            that is known to the bot and was edited.
+            that is known to the bot and was edited. This update may at times be triggered by
+            changes to message fields that are either unavailable or not actively used by your bot.
         inline_query (:class:`telegram.InlineQuery`): Optional. New incoming inline query.
         chosen_inline_result (:class:`telegram.ChosenInlineResult`): Optional. The result of an
             inline query that was chosen by a user and sent to their chat partner.
@@ -123,7 +163,7 @@ class Update(TelegramObject):
         pre_checkout_query (:class:`telegram.PreCheckoutQuery`): Optional. New incoming
             pre-checkout query. Contains full information about checkout.
         poll (:class:`telegram.Poll`): Optional. New poll state. Bots receive only updates about
-            stopped polls and polls, which are sent by the bot.
+            manually stopped polls and polls, which are sent by the bot.
         poll_answer (:class:`telegram.PollAnswer`): Optional. A user changed their answer
             in a non-anonymous poll. Bots receive new votes only in polls that were sent
             by the bot itself.
@@ -148,86 +188,137 @@ class Update(TelegramObject):
 
             .. versionadded:: 13.8
 
+        chat_boost (:class:`telegram.ChatBoostUpdated`): Optional. A chat boost was added or
+            changed. The bot must be an administrator in the chat to receive these updates.
+
+            .. versionadded:: 20.8
+
+        removed_chat_boost (:class:`telegram.ChatBoostRemoved`): Optional. A boost was removed from
+            a chat. The bot must be an administrator in the chat to receive these updates.
+
+            .. versionadded:: 20.8
+
+        message_reaction (:class:`telegram.MessageReactionUpdated`): Optional. A reaction to a
+            message was changed by a user. The bot must be an administrator in the chat and must
+            explicitly specify :attr:`MESSAGE_REACTION` in the list of
+            :paramref:`telegram.ext.Application.run_polling.allowed_updates` to receive these
+            updates (see :meth:`telegram.Bot.get_updates`, :meth:`telegram.Bot.set_webhook`,
+            :meth:`telegram.ext.Application.run_polling` and
+            :meth:`telegram.ext.Application.run_webhook`). The update isn't received for reactions
+            set by bots.
+
+            .. versionadded:: 20.8
+
+        message_reaction_count (:class:`telegram.MessageReactionCountUpdated`): Optional. Reactions
+            to a message with anonymous reactions were changed. The bot must be an administrator in
+            the chat and must explicitly specify :attr:`MESSAGE_REACTION_COUNT` in the list of
+            :paramref:`telegram.ext.Application.run_polling.allowed_updates` to receive these
+            updates (see :meth:`telegram.Bot.get_updates`, :meth:`telegram.Bot.set_webhook`,
+            :meth:`telegram.ext.Application.run_polling` and
+            :meth:`telegram.ext.Application.run_webhook`). The updates are grouped and can be sent
+            with delay up to a few minutes.
+
+            .. versionadded:: 20.8
     """
 
     __slots__ = (
-        "callback_query",
-        "chosen_inline_result",
-        "pre_checkout_query",
-        "inline_query",
-        "update_id",
-        "message",
-        "shipping_query",
-        "poll",
-        "poll_answer",
-        "channel_post",
-        "edited_channel_post",
-        "edited_message",
-        "_effective_user",
         "_effective_chat",
         "_effective_message",
-        "my_chat_member",
-        "chat_member",
+        "_effective_user",
+        "callback_query",
+        "channel_post",
+        "chat_boost",
         "chat_join_request",
+        "chat_member",
+        "chosen_inline_result",
+        "edited_channel_post",
+        "edited_message",
+        "inline_query",
+        "message",
+        "message_reaction",
+        "message_reaction_count",
+        "my_chat_member",
+        "poll",
+        "poll_answer",
+        "pre_checkout_query",
+        "removed_chat_boost",
+        "shipping_query",
+        "update_id",
     )
 
-    MESSAGE: ClassVar[str] = constants.UpdateType.MESSAGE
+    MESSAGE: Final[str] = constants.UpdateType.MESSAGE
     """:const:`telegram.constants.UpdateType.MESSAGE`
 
     .. versionadded:: 13.5"""
-    EDITED_MESSAGE: ClassVar[str] = constants.UpdateType.EDITED_MESSAGE
+    EDITED_MESSAGE: Final[str] = constants.UpdateType.EDITED_MESSAGE
     """:const:`telegram.constants.UpdateType.EDITED_MESSAGE`
 
     .. versionadded:: 13.5"""
-    CHANNEL_POST: ClassVar[str] = constants.UpdateType.CHANNEL_POST
+    CHANNEL_POST: Final[str] = constants.UpdateType.CHANNEL_POST
     """:const:`telegram.constants.UpdateType.CHANNEL_POST`
 
     .. versionadded:: 13.5"""
-    EDITED_CHANNEL_POST: ClassVar[str] = constants.UpdateType.EDITED_CHANNEL_POST
+    EDITED_CHANNEL_POST: Final[str] = constants.UpdateType.EDITED_CHANNEL_POST
     """:const:`telegram.constants.UpdateType.EDITED_CHANNEL_POST`
 
     .. versionadded:: 13.5"""
-    INLINE_QUERY: ClassVar[str] = constants.UpdateType.INLINE_QUERY
+    INLINE_QUERY: Final[str] = constants.UpdateType.INLINE_QUERY
     """:const:`telegram.constants.UpdateType.INLINE_QUERY`
 
     .. versionadded:: 13.5"""
-    CHOSEN_INLINE_RESULT: ClassVar[str] = constants.UpdateType.CHOSEN_INLINE_RESULT
+    CHOSEN_INLINE_RESULT: Final[str] = constants.UpdateType.CHOSEN_INLINE_RESULT
     """:const:`telegram.constants.UpdateType.CHOSEN_INLINE_RESULT`
 
     .. versionadded:: 13.5"""
-    CALLBACK_QUERY: ClassVar[str] = constants.UpdateType.CALLBACK_QUERY
+    CALLBACK_QUERY: Final[str] = constants.UpdateType.CALLBACK_QUERY
     """:const:`telegram.constants.UpdateType.CALLBACK_QUERY`
 
     .. versionadded:: 13.5"""
-    SHIPPING_QUERY: ClassVar[str] = constants.UpdateType.SHIPPING_QUERY
+    SHIPPING_QUERY: Final[str] = constants.UpdateType.SHIPPING_QUERY
     """:const:`telegram.constants.UpdateType.SHIPPING_QUERY`
 
     .. versionadded:: 13.5"""
-    PRE_CHECKOUT_QUERY: ClassVar[str] = constants.UpdateType.PRE_CHECKOUT_QUERY
+    PRE_CHECKOUT_QUERY: Final[str] = constants.UpdateType.PRE_CHECKOUT_QUERY
     """:const:`telegram.constants.UpdateType.PRE_CHECKOUT_QUERY`
 
     .. versionadded:: 13.5"""
-    POLL: ClassVar[str] = constants.UpdateType.POLL
+    POLL: Final[str] = constants.UpdateType.POLL
     """:const:`telegram.constants.UpdateType.POLL`
 
     .. versionadded:: 13.5"""
-    POLL_ANSWER: ClassVar[str] = constants.UpdateType.POLL_ANSWER
+    POLL_ANSWER: Final[str] = constants.UpdateType.POLL_ANSWER
     """:const:`telegram.constants.UpdateType.POLL_ANSWER`
 
     .. versionadded:: 13.5"""
-    MY_CHAT_MEMBER: ClassVar[str] = constants.UpdateType.MY_CHAT_MEMBER
+    MY_CHAT_MEMBER: Final[str] = constants.UpdateType.MY_CHAT_MEMBER
     """:const:`telegram.constants.UpdateType.MY_CHAT_MEMBER`
 
     .. versionadded:: 13.5"""
-    CHAT_MEMBER: ClassVar[str] = constants.UpdateType.CHAT_MEMBER
+    CHAT_MEMBER: Final[str] = constants.UpdateType.CHAT_MEMBER
     """:const:`telegram.constants.UpdateType.CHAT_MEMBER`
 
     .. versionadded:: 13.5"""
-    CHAT_JOIN_REQUEST = constants.UpdateType.CHAT_JOIN_REQUEST
+    CHAT_JOIN_REQUEST: Final[str] = constants.UpdateType.CHAT_JOIN_REQUEST
     """:const:`telegram.constants.UpdateType.CHAT_JOIN_REQUEST`
 
     .. versionadded:: 13.8"""
-    ALL_TYPES: ClassVar[List[str]] = list(constants.UpdateType)
+    CHAT_BOOST: Final[str] = constants.UpdateType.CHAT_BOOST
+    """:const:`telegram.constants.UpdateType.CHAT_BOOST`
+
+    .. versionadded:: 20.8"""
+    REMOVED_CHAT_BOOST: Final[str] = constants.UpdateType.REMOVED_CHAT_BOOST
+    """:const:`telegram.constants.UpdateType.REMOVED_CHAT_BOOST`
+
+    .. versionadded:: 20.8"""
+    MESSAGE_REACTION: Final[str] = constants.UpdateType.MESSAGE_REACTION
+    """:const:`telegram.constants.UpdateType.MESSAGE_REACTION`
+
+    .. versionadded:: 20.8"""
+    MESSAGE_REACTION_COUNT: Final[str] = constants.UpdateType.MESSAGE_REACTION_COUNT
+    """:const:`telegram.constants.UpdateType.MESSAGE_REACTION_COUNT`
+
+    .. versionadded:: 20.8"""
+    ALL_TYPES: Final[List[str]] = list(constants.UpdateType)
     """List[:obj:`str`]: A list of all available update types.
 
     .. versionadded:: 13.5"""
@@ -235,22 +326,26 @@ class Update(TelegramObject):
     def __init__(
         self,
         update_id: int,
-        message: Message = None,
-        edited_message: Message = None,
-        channel_post: Message = None,
-        edited_channel_post: Message = None,
-        inline_query: InlineQuery = None,
-        chosen_inline_result: ChosenInlineResult = None,
-        callback_query: CallbackQuery = None,
-        shipping_query: ShippingQuery = None,
-        pre_checkout_query: PreCheckoutQuery = None,
-        poll: Poll = None,
-        poll_answer: PollAnswer = None,
-        my_chat_member: ChatMemberUpdated = None,
-        chat_member: ChatMemberUpdated = None,
-        chat_join_request: ChatJoinRequest = None,
+        message: Optional[Message] = None,
+        edited_message: Optional[Message] = None,
+        channel_post: Optional[Message] = None,
+        edited_channel_post: Optional[Message] = None,
+        inline_query: Optional[InlineQuery] = None,
+        chosen_inline_result: Optional[ChosenInlineResult] = None,
+        callback_query: Optional[CallbackQuery] = None,
+        shipping_query: Optional[ShippingQuery] = None,
+        pre_checkout_query: Optional[PreCheckoutQuery] = None,
+        poll: Optional[Poll] = None,
+        poll_answer: Optional[PollAnswer] = None,
+        my_chat_member: Optional[ChatMemberUpdated] = None,
+        chat_member: Optional[ChatMemberUpdated] = None,
+        chat_join_request: Optional[ChatJoinRequest] = None,
+        chat_boost: Optional[ChatBoostUpdated] = None,
+        removed_chat_boost: Optional[ChatBoostRemoved] = None,
+        message_reaction: Optional[MessageReactionUpdated] = None,
+        message_reaction_count: Optional[MessageReactionCountUpdated] = None,
         *,
-        api_kwargs: JSONDict = None,
+        api_kwargs: Optional[JSONDict] = None,
     ):
         super().__init__(api_kwargs=api_kwargs)
         # Required
@@ -270,9 +365,13 @@ class Update(TelegramObject):
         self.my_chat_member: Optional[ChatMemberUpdated] = my_chat_member
         self.chat_member: Optional[ChatMemberUpdated] = chat_member
         self.chat_join_request: Optional[ChatJoinRequest] = chat_join_request
+        self.chat_boost: Optional[ChatBoostUpdated] = chat_boost
+        self.removed_chat_boost: Optional[ChatBoostRemoved] = removed_chat_boost
+        self.message_reaction: Optional[MessageReactionUpdated] = message_reaction
+        self.message_reaction_count: Optional[MessageReactionCountUpdated] = message_reaction_count
 
-        self._effective_user: Optional["User"] = None
-        self._effective_chat: Optional["Chat"] = None
+        self._effective_user: Optional[User] = None
+        self._effective_chat: Optional[Chat] = None
         self._effective_message: Optional[Message] = None
 
         self._id_attrs = (self.update_id,)
@@ -284,7 +383,16 @@ class Update(TelegramObject):
         """
         :class:`telegram.User`: The user that sent this update, no matter what kind of update this
         is. If no user is associated with this update, this gives :obj:`None`. This is the case
-        if :attr:`channel_post`, :attr:`edited_channel_post` or :attr:`poll` is present.
+        if any of
+
+        * :attr:`channel_post`
+        * :attr:`edited_channel_post`
+        * :attr:`poll`
+        * :attr:`chat_boost`
+        * :attr:`removed_chat_boost`
+        * :attr:`message_reaction_count`
+
+        is present.
 
         Example:
             * If :attr:`message` is present, this will give
@@ -330,6 +438,9 @@ class Update(TelegramObject):
         elif self.chat_join_request:
             user = self.chat_join_request.from_user
 
+        elif self.message_reaction:
+            user = self.message_reaction.user
+
         self._effective_user = user
         return user
 
@@ -377,6 +488,18 @@ class Update(TelegramObject):
         elif self.chat_join_request:
             chat = self.chat_join_request.chat
 
+        elif self.chat_boost:
+            chat = self.chat_boost.chat
+
+        elif self.removed_chat_boost:
+            chat = self.removed_chat_boost.chat
+
+        elif self.message_reaction:
+            chat = self.message_reaction.chat
+
+        elif self.message_reaction_count:
+            chat = self.message_reaction_count.chat
+
         self._effective_chat = chat
         return chat
 
@@ -389,11 +512,18 @@ class Update(TelegramObject):
             :attr:`callback_query` (i.e. :attr:`telegram.CallbackQuery.message`) or :obj:`None`, if
             none of those are present.
 
+        Tip:
+            This property will only ever return objects of type :class:`telegram.Message` or
+            :obj:`None`, never :class:`telegram.MaybeInaccessibleMessage` or
+            :class:`telegram.InaccessibleMessage`.
+            Currently, this is only relevant for :attr:`callback_query`, as
+            :attr:`telegram.CallbackQuery.message` is the only attribute considered by this
+            property that can be an object of these types.
         """
         if self._effective_message:
             return self._effective_message
 
-        message = None
+        message: Optional[Message] = None
 
         if self.message:
             message = self.message
@@ -402,7 +532,21 @@ class Update(TelegramObject):
             message = self.edited_message
 
         elif self.callback_query:
-            message = self.callback_query.message
+            if (
+                isinstance(cbq_message := self.callback_query.message, Message)
+                or cbq_message is None
+            ):
+                message = cbq_message
+            else:
+                warn(
+                    (
+                        "`update.callback_query` is not `None`, but of type "
+                        f"`{cbq_message.__class__.__name__}`. This is not considered by "
+                        "`Update.effective_message`. Please manually access this attribute "
+                        "if necessary."
+                    ),
+                    stacklevel=2,
+                )
 
         elif self.channel_post:
             message = self.channel_post
@@ -437,5 +581,13 @@ class Update(TelegramObject):
         data["my_chat_member"] = ChatMemberUpdated.de_json(data.get("my_chat_member"), bot)
         data["chat_member"] = ChatMemberUpdated.de_json(data.get("chat_member"), bot)
         data["chat_join_request"] = ChatJoinRequest.de_json(data.get("chat_join_request"), bot)
+        data["chat_boost"] = ChatBoostUpdated.de_json(data.get("chat_boost"), bot)
+        data["removed_chat_boost"] = ChatBoostRemoved.de_json(data.get("removed_chat_boost"), bot)
+        data["message_reaction"] = MessageReactionUpdated.de_json(
+            data.get("message_reaction"), bot
+        )
+        data["message_reaction_count"] = MessageReactionCountUpdated.de_json(
+            data.get("message_reaction_count"), bot
+        )
 
         return super().de_json(data=data, bot=bot)
